/*
    Aufgabe 5) Zweidimensionale Arrays und Rekursion - Minesweeper
*/
import java.awt.*;
public class  Aufgabe5 {

    private static int mineCounter = 0;
    private static int openFields = 0;

    // method generates a mine board
    private static boolean[][] genMineBoard(int bSize, float p){
        boolean[][] mineBoard = new boolean[bSize + 2][bSize + 2];
        for (int y = 1; y <= bSize; y++) {
            for (int x = 1; x <= bSize; x++) {
                if(Math.random() < p) {
                    mineBoard[y][x] = true;
                    mineCounter++;
                }
            }
        }
        return mineBoard;
    }

    // method calculates for each position how many mines are around
    // and generates a new integer array
    private static int[][] calcGameBoard(int bSize, boolean[][] mineBoard){
        int[][] gameBoard = new int[bSize + 2][bSize + 2];
        for (int y = 1; y <= bSize; y++){
            for (int x = 1; x <= bSize; x++){
                for (int wy = y - 1; wy <= y + 1; wy++){
                    for (int wx = x - 1; wx <= x + 1; wx++){
                        if (mineBoard[wy][wx]){
                            gameBoard[y][x]++;
                        }
                    }
                }
            }
        }
        return gameBoard;
    }

    // method which draws the current status of the game board
    private static void drawGameBoard(int width, int height, int bSize, boolean[][] mineBoard, int[][] gameBoard, boolean[][] clickBoard){
        int cy = bSize;
        int cx = 1;
        for (int y = (height/bSize)/2; y < height; y+=height/bSize){
            for (int x = (width/bSize)/2; x < width; x+=width/bSize){
                StdDraw.setPenColor(StdDraw.BLUE);
                StdDraw.filledSquare(x,y,(width/bSize)/2);
                StdDraw.setPenColor(StdDraw.BLACK);
                StdDraw.square(x,y,(width/bSize)/2);

                if(clickBoard[cy-1][cx-1]){
                    if(mineBoard[cy][cx]){
                        StdDraw.setPenColor(StdDraw.RED);
                        StdDraw.filledCircle(x,y,(width/bSize)/4);
                    }
                    else{
                        StdDraw.setPenColor(StdDraw.YELLOW);
                        StdDraw.text(x, y, Integer.toString(gameBoard[cy][cx]));
                    }
                }
                else{
                    StdDraw.setPenColor(StdDraw.GRAY);
                    StdDraw.filledSquare(x,y,(width/bSize)/2);
                    StdDraw.setPenColor(StdDraw.BLACK);
                    StdDraw.square(x,y,(width/bSize)/2);
                }
                cx++;
            }
            cy--;
            cx = 1;
        }
        StdDraw.show();
    }

    private static void findZeroFields(int x, int y, int bSize, boolean[][] mineBoard, int[][] gameBoard, boolean[][] clickBoard) {
        // TODO: Implementieren Sie hier Ihre Lösung für die Methode
    }


    private static boolean checkNewClick(int x, int y, int bSize, boolean[][] mineBoard, int[][] gameBoard, boolean[][] clickBoard){
        // TODO: Implementieren Sie hier Ihre Lösung für die Methode
        return false;
    }

    // helping method for debugging
    private static void printBoard(boolean[][] mineBoard){
        for (int y = 1; y < mineBoard.length-1; y++){
            System.out.println();
            for (int x = 1; x < mineBoard[y].length-1; x++){
                if(mineBoard[y][x]){
                    System.out.print("* ");
                }
                else{
                    System.out.print("O ");
                }
            }
        }
        System.out.println();
    }

    public static void main(String[] args) {
        
        // canvas settings
        final int height = 400;
        final int width = 400;
        StdDraw.setCanvasSize(width, height);
        StdDraw.setScale(0, 400);
        Font font = new Font("Arial", Font.BOLD, 15);
        StdDraw.setFont(font);
        
        // game variables
        int bSize = 10;
        float p = 0.2f;
        boolean[][] mineBoard = genMineBoard(bSize,p);
        printBoard(mineBoard); // for debugging
        int[][] gameBoard = calcGameBoard(bSize,mineBoard);
        boolean[][] clickBoard = new boolean[bSize][bSize];
        boolean isGameOver = false;
        boolean hasWon = false;

        StdDraw.enableDoubleBuffering();
        
        // initial draw of game board
        drawGameBoard(width, height, bSize, mineBoard, gameBoard, clickBoard);
        
        // game play starts
        while(!isGameOver && !hasWon){
            if(StdDraw.isMousePressed()){
                // check new click if it is a mine and open all connected zero fields
                isGameOver = checkNewClick(((int)StdDraw.mouseX())/(width/bSize), ((int)StdDraw.mouseY())/(height/bSize), bSize, mineBoard, gameBoard, clickBoard);
                drawGameBoard(width, height, bSize, mineBoard, gameBoard, clickBoard);
                if((bSize*bSize - openFields) == mineCounter && !isGameOver){
                    // all fields open - all mines retrieved - player won
                    font = new Font("Arial", Font.BOLD, 50);
                    StdDraw.setFont(font);
                    StdDraw.setPenColor(StdDraw.GREEN);
                    StdDraw.text(width/2, height/2, "YOU WON !!!");
                    hasWon = true;
                }
                if(isGameOver){
                    // current click was a mine - game over
                    font = new Font("Arial", Font.BOLD, 50);
                    StdDraw.setFont(font);
                    StdDraw.setPenColor(StdDraw.RED);
                    StdDraw.text(width/2, height/2, "GAME OVER!");
                }
                StdDraw.show();
                
                // status outputs
                System.out.println("x: "+StdDraw.mouseX()+" y: "+StdDraw.mouseY());
                System.out.println("bombs: " + mineCounter);
                System.out.println("open: " + openFields);
            }
            StdDraw.pause(50);
        }
    }
}

